require "gauss/version"
require "statsd"
require 'benchmark'
require 'pry'

module Gauss
  class << self
    attr_accessor :prefix, :metrics, :mode
  end

  def self.included(k)
    begin
      self.prefix ||= Rails.application.class.parent_name
    rescue
    end

    begin
      Gauss::metrics ||= Statsd.new(ENV["STATSD_SERVER"], ENV["STATSD_SERVER_PORT"] || 8125)
    rescue
    end
  end

  def self.server=(server, port=8125)
      Gauss::metrics = Statsd.new(server, port)
  end

  def self.benchmark_prefix=(prefix)
    self.prefix = prefix
  end

  def benchmark(suffix=nil, &block)
    name = ""
    name += "#{Strings::underscore(Gauss::prefix)}." if Gauss::prefix
    name += "#{Strings::underscore(self.class.name)}.#{caller[0][/`([^']*)'/, 1]}"
    name += "-#{Strings::underscore(suffix)}" if suffix
    
    if Gauss::mode.to_sym == :local
      r = nil
      m = Benchmark.measure { r = yield }
      p "#{name}: #{m}"
      r
    else

      Gauss::metrics.time(name) { yield }
    end
  end


  module Strings
    def self.underscore(s)
      s.gsub(/::/, '/').
        gsub(/([A-Z]+)([A-Z][a-z])/,'\1_\2').
        gsub(/([a-z\d])([A-Z])/,'\1_\2').
        tr("-", "_").
        downcase
    end
  end
end
